######################################################################
#
# File: b2sdk/_internal/testing/helpers/base.py
#
# Copyright 2022 Backblaze Inc. All Rights Reserved.
#
# License https://www.backblaze.com/using_b2_code.html
#
######################################################################
from __future__ import annotations

import pytest

from b2sdk._internal.testing.helpers.bucket_manager import BucketManager
from b2sdk.v2 import B2Api


@pytest.mark.usefixtures('cls_setup')
class IntegrationTestBase:
    b2_api: B2Api
    this_run_bucket_name_prefix: str
    bucket_manager: BucketManager

    @pytest.fixture(autouse=True, scope='class')
    def cls_setup(self, request, b2_api, b2_auth_data, bucket_name_prefix, bucket_manager):
        cls = request.cls
        cls.b2_auth_data = b2_auth_data
        cls.this_run_bucket_name_prefix = bucket_name_prefix
        cls.bucket_manager = bucket_manager
        cls.b2_api = b2_api
        cls.info = b2_api.account_info

    @pytest.fixture(autouse=True)
    def setup_method(self):
        self.buckets_created = []
        yield
        for bucket in self.buckets_created:
            self.bucket_manager.clean_bucket(bucket)

    def write_zeros(self, file, number):
        line = b'0' * 1000 + b'\n'
        line_len = len(line)
        written = 0
        while written <= number:
            file.write(line)
            written += line_len

    def create_bucket(self):
        bucket = self.bucket_manager.create_bucket()
        self.buckets_created.append(bucket)
        return bucket



######################################################################
#
# File: b2sdk/v2/account_info.py
#
# Copyright 2023 Backblaze Inc. All Rights Reserved.
#
# License https://www.backblaze.com/using_b2_code.html
#
######################################################################
from __future__ import annotations


from copy import copy
from b2sdk import v3
from .exception import MissingAccountData


class _OldAllowedMixin:
    DEFAULT_ALLOWED = dict(
        bucketId=None,
        bucketName=None,
        capabilities=v3.ALL_CAPABILITIES,
        namePrefix=None,
    )

    @classmethod
    def allowed_is_valid(cls, allowed):
        return (
            ('bucketId' in allowed)
            and ('bucketName' in allowed)
            and ((allowed['bucketId'] is not None) or (allowed['bucketName'] is None))
            and ('capabilities' in allowed)
            and ('namePrefix' in allowed)
        )

    @classmethod
    def _convert_allowed_single_to_multi_bucket(cls, allowed):
        new = copy(allowed)

        bucket_id = new.pop('bucketId')
        bucket_name = new.pop('bucketName')

        if bucket_id is not None:
            new['buckets'] = [{'id': bucket_id, 'name': bucket_name}]
        else:
            new['buckets'] = None

        return new

    def get_allowed(self):
        allowed = super().get_allowed()

        if 'buckets' in allowed:
            buckets = allowed.pop('buckets')
            if buckets and len(buckets) > 1:
                raise MissingAccountData(
                    'Multi-bucket keys cannot be used with the current sdk version'
                )

            allowed['bucketId'] = buckets[0]['id'] if buckets else None
            allowed['bucketName'] = buckets[0]['name'] if buckets else None

        return allowed

    def _set_auth_data(
        self,
        account_id,
        auth_token,
        api_url,
        download_url,
        recommended_part_size,
        absolute_minimum_part_size,
        application_key,
        realm,
        s3_api_url,
        allowed,
        application_key_id,
    ):
        new_allowed = self._convert_allowed_single_to_multi_bucket(allowed)

        super()._set_auth_data(
            account_id,
            auth_token,
            api_url,
            download_url,
            recommended_part_size,
            absolute_minimum_part_size,
            application_key,
            realm,
            s3_api_url,
            new_allowed,
            application_key_id,
        )


class AbstractAccountInfo(_OldAllowedMixin, v3.AbstractAccountInfo):
    def list_bucket_names_ids(self):
        return []  # Removed @abstractmethod decorator


class UrlPoolAccountInfo(_OldAllowedMixin, v3.UrlPoolAccountInfo):
    pass


class InMemoryAccountInfo(_OldAllowedMixin, v3.InMemoryAccountInfo):
    pass


class SqliteAccountInfo(_OldAllowedMixin, v3.SqliteAccountInfo):
    pass


class StubAccountInfo(_OldAllowedMixin, v3.StubAccountInfo):
    pass



######################################################################
#
# File: b2sdk/_internal/account_info/exception.py
#
# Copyright 2019 Backblaze Inc. All Rights Reserved.
#
# License https://www.backblaze.com/using_b2_code.html
#
######################################################################
from __future__ import annotations

from abc import ABCMeta

from ..exception import B2Error


class AccountInfoError(B2Error, metaclass=ABCMeta):
    """
    Base class for all account info errors.
    """

    pass


class CorruptAccountInfo(AccountInfoError):
    """
    Raised when an account info file is corrupted.
    """

    def __init__(self, file_name):
        """
        :param file_name: an account info file name
        :type file_name: str
        """
        super().__init__()
        self.file_name = file_name

    def __str__(self):
        return (
            f'Account info file ({self.file_name}) appears corrupted. '
            f'Try removing and then re-authorizing the account.'
        )


class MissingAccountData(AccountInfoError):
    """
    Raised when there is no account info data available.
    """

    def __init__(self, key):
        """
        :param key: a key for getting account data
        :type key: str
        """
        super().__init__()
        self.key = key

    def __str__(self):
        return f'Missing account data: {self.key}'



######################################################################
#
# File: b2sdk/_internal/testing/fixtures/account_info.py
#
# Copyright 2025 Backblaze Inc. All Rights Reserved.
#
# License https://www.backblaze.com/using_b2_code.html
#
######################################################################

import pytest

from b2sdk._internal.account_info.sqlite_account_info import (
    B2_ACCOUNT_INFO_ENV_VAR,
    XDG_CONFIG_HOME_ENV_VAR,
)


@pytest.fixture(scope='session')
def change_account_info_dir(monkeysession, tmp_path_factory):
    """
    For the entire test session:
    1) temporarily remove B2_APPLICATION_KEY and B2_APPLICATION_KEY_ID from the environment
    2) create a temporary directory for storing account info database
    3) set B2_ACCOUNT_INFO_ENV_VAR to point to the temporary account info file
    """

    monkeysession.delenv('B2_APPLICATION_KEY_ID', raising=False)
    monkeysession.delenv('B2_APPLICATION_KEY', raising=False)

    temp_dir = tmp_path_factory.mktemp('b2_config')

    monkeysession.setenv(B2_ACCOUNT_INFO_ENV_VAR, str(temp_dir / ('.b2_account_info')))
    monkeysession.setenv(XDG_CONFIG_HOME_ENV_VAR, str(temp_dir))



######################################################################
#
# File: b2sdk/_internal/testing/fixtures/common.py
#
# Copyright 2025 Backblaze Inc. All Rights Reserved.
#
# License https://www.backblaze.com/using_b2_code.html
#
######################################################################
import pytest


@pytest.fixture(scope='session')
def monkeysession():
    with pytest.MonkeyPatch.context() as mp:
        yield mp
